<?php

/*
 * Copyright (C) 2015 IRSTEA
 * All rights reserved.
 */

namespace Irstea\FileUploadBundle\Form\Type;

use Irstea\FileUploadBundle\Controller\UploadController;
use Irstea\FileUploadBundle\Form\DataTranformer\UploadedFileTransformer;
use Irstea\FileUploadBundle\Service\FileManagerInterface;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Csrf\CsrfProvider\CsrfProviderInterface;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormView;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Routing\Router;

class FileUploadType extends AbstractType
{
    use \Irstea\ThemeBundle\Form\AbstractJQueryWidgetTrait;

    /**
     *
     * @var Router
     */
    private $router;

    /**
     *
     * @var FileManagerInterface
     */
    private $fileManager;

    /**
     * @var CsrfProviderInterface
     */
    protected $csrfProvider;

    /**
     *
     * @param Router $router
     * @param FileManagerInterface $fileManager
     */
    public function __construct(Router $router, FileManagerInterface $fileManager, CsrfProviderInterface $csrfProvider)
    {
        $this->router = $router;
        $this->fileManager = $fileManager;
        $this->csrfProvider = $csrfProvider;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder->setCompound(false);
        $builder->addViewTransformer(new UploadedFileTransformer($this->fileManager, $options['multiple']));
    }

    public function buildView(FormView $view, FormInterface $form, array $options)
    {
        $this->buildWidgetView($view, $form, $options);

        $view->vars['widget_attr']['data-create-url'] = $this->router->generate('file_upload_create');
        $view->vars['csrfToken'] = $this->csrfProvider->generateCsrfToken(UploadController::CSRF_INTENTION);
        $view->vars['multiple'] = $options['multiple'];
    }

    protected function getWidgetDefaults()
    {
        return [
            'multiple'             => false,
            'accept_file_types'    => '',
            'max_file_size'        => 0,
            'min_file_size'        => 0,
            'max_number_of_files'  => 0,
            'bitrate_interval'     => 512,
            'progress_interval'    => 100,
            'recalculate_progress' => true,
            'max_chunk_size'       => 1000000,
            'sequential_uploads'   => false,
        ];
    }

    public function getName()
    {
        return "file_upload";
    }

    protected function getWidgetType()
    {
        return "irsteaFileUpload";
    }
}
